//+------------------------------------------------------------------+
//|                                                         MACD.mq4 |
//|                                Copyright  2005, David W. Thomas |
//|                                           mailto:davidwt@usa.net |
//|                               modified by Linuxser for Forex TSD |
//|                                        to look like eSignal MACD |
//|added OSMA multicolor, Magnifier, Arrrows by brax64 for Forex TSD |
//+------------------------------------------------------------------+

// This is the correct computation and display of MACD. 24th July 2012

#property copyright "Copyright  2005, David W. Thomas" //NB 
#property link      "mailto:davidwt@usa.net"

#property indicator_separate_window
#property indicator_buffers 6
#property indicator_color1 C'10,20,30'
#property indicator_color2 Gold//DimGray
#property indicator_color3 C'35,95,235'
#property indicator_color4 C'200,0,0'
#property indicator_color5 WhiteSmoke
#property indicator_color6 CLR_NONE

#property indicator_width1 5
#property indicator_width2 1
#property indicator_width3 3
#property indicator_width4 3
#property indicator_width5 0
#property indicator_width6 0
#property indicator_style2 2

//---- input parameters -------------------------

extern double    FastMAPeriod     = 9.0503; //8.3896; ek het dit omgeruil vir beter lesings
extern double    SlowMAPeriod     = 17.5185;
extern double    SignalMAPeriod   = 8.3896; //9.0503;
extern double    Magnifier        = 1.5;
extern bool      ShowArrows       = true;//************************
extern string    arrowsIdentifier = "eMACD arrows";
extern color     arrowsUpColor    = Aqua;//DeepSkyBlue;
extern color     arrowsDnColor    = Yellow;//Red;
extern double    gapMultiplierUpper  = 1;
extern double    gapMultiplierLower  = 1;

//---- buffers --------------

double MACDLineBuffer[];
double SignalLineBuffer[];
double HistogramBufferUP[];
double HistogramBufferDN[];
double MACDHistoBuffer[];
double HistogramBuffer[];

//---- variables

double alpha = 0;
double alpha_1 = 0;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+

int init()

{
   IndicatorDigits(Digits + 2);
 
   SetIndexStyle(0,DRAW_HISTOGRAM); SetIndexBuffer(0,MACDHistoBuffer);   SetIndexDrawBegin(0,SlowMAPeriod);
   SetIndexStyle(1,DRAW_LINE);      SetIndexBuffer(1,SignalLineBuffer);  SetIndexDrawBegin(1,SlowMAPeriod+SignalMAPeriod);
   SetIndexStyle(2,DRAW_HISTOGRAM); SetIndexBuffer(2,HistogramBufferUP); SetIndexDrawBegin(2,SlowMAPeriod+SignalMAPeriod);
   SetIndexStyle(3,DRAW_HISTOGRAM); SetIndexBuffer(3,HistogramBufferDN); SetIndexDrawBegin(3,SlowMAPeriod+SignalMAPeriod);
   SetIndexStyle(4,DRAW_LINE);      SetIndexBuffer(4,MACDLineBuffer);    SetIndexDrawBegin(4,SlowMAPeriod);
   SetIndexStyle(5,DRAW_LINE);      SetIndexBuffer(5,HistogramBuffer);
   
   IndicatorShortName("eMACD v1.1 ("+DoubleToStr(FastMAPeriod,2)+","+DoubleToStr(SlowMAPeriod,2)+","+DoubleToStr(SignalMAPeriod,2)+")");
   SetIndexLabel(0,"MACD");
   SetIndexLabel(1,"Signal");
   
   FastMAPeriod   = MathMax(FastMAPeriod,1);
   SlowMAPeriod   = MathMax(SlowMAPeriod,1);
   SignalMAPeriod = MathMax(SignalMAPeriod,1);
   
	alpha = 2.0 / (SignalMAPeriod + 1.0);
	alpha_1 = 1.0 - alpha;

   return(0);
}


//+------------------------------------------------------------------+
//| Custor indicator deinitialization function                       |
//+------------------------------------------------------------------+


int deinit()
{
   string lookFor       = arrowsIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

int start()
{   
   int limit,i,counted_bars=IndicatorCounted();

   if(counted_bars < 0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit  = MathMin(Bars-counted_bars,Bars-1);

   for(i=limit; i>=0; i--)

   {
      MACDLineBuffer[i]    = iMA(NULL,0,FastMAPeriod,0,MODE_EMA,PRICE_CLOSE,i) - iMA(NULL,0,SlowMAPeriod,0,MODE_EMA,PRICE_CLOSE,i);
      SignalLineBuffer[i]  = alpha*MACDLineBuffer[i] + alpha_1*SignalLineBuffer[i+1];
      MACDHistoBuffer[i]   = MACDLineBuffer[i];
      
      HistogramBuffer[i]   = (MACDLineBuffer[i] - SignalLineBuffer[i]) * Magnifier;
      HistogramBufferUP[i] = EMPTY_VALUE;
      HistogramBufferDN[i] = EMPTY_VALUE;
           
      if (HistogramBuffer[i] > 0)
      
            HistogramBufferUP[i] = HistogramBuffer[i];
      else  HistogramBufferDN[i] = HistogramBuffer[i];
     
      manageArrow(i);
         
   }
   
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

void manageArrow(int i)
{
   if (ShowArrows)
   {
      deleteArrow(Time[i]);
      {
         if (HistogramBuffer[i] > 0) drawArrow(i,arrowsUpColor,241);
         else                        drawArrow(i,arrowsDnColor,242);
      }
   }
}               

void drawArrow(int i,color theColor,int theCode)
{
   
   string name = arrowsIdentifier+":"+Time[i];
   double gap  = iATR(NULL,0,20,i); //****************** 0,20,i
      
      ObjectCreate(name,OBJ_ARROW,0,Time[i],0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,theColor);
  
         if       (HistogramBuffer[i] > 0 && HistogramBuffer[i+1] < 0)
                             ObjectSet(name,OBJPROP_PRICE1,Low[i]-gap*gapMultiplierLower);
         else if (HistogramBuffer[i] < 0 && HistogramBuffer[i+1] > 0)
                             ObjectSet(name,OBJPROP_PRICE1,High[i]+gap*gapMultiplierUpper);
}

void deleteArrow(datetime time)
{
   string lookFor = arrowsIdentifier+":"+time; ObjectDelete(lookFor);
} 
//==================================================================+